@@ -26,13 +26,11 @@ void read_state_file (struct tgl_state *TLS);
 void read_auth_file (struct tgl_state *TLS);
 void write_auth_file (struct tgl_state *TLS);
 void write_state_file (struct tgl_state *TLS);
+void write_state_file_schedule (struct tgl_state *TLS);
 void read_secret_chat_file (struct tgl_state *TLS);
 void write_secret_chat_file (struct tgl_state *TLS);
 
 void telegram_login (struct tgl_state *TLS);
-PurpleConversation *chat_show (PurpleConnection *gc, int id);
-int chat_add_message (struct tgl_state *TLS, struct tgl_message *M, char *text);
-void chat_add_all_users (PurpleConversation *pc, struct tgl_chat *chat);
 void request_code_entered (gpointer data, const gchar *code);
 int generate_ident_icon(struct tgl_state *TLS, unsigned char* sha1_key);
 
