@@ -15,7 +15,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Copyright Vitaly Valtman 2013-2014
+    Copyright Vitaly Valtman 2013-2015
 */
 #include "tgl.h"
 #include "updates.h"
@@ -431,6 +431,18 @@ void tglu_work_update (struct tgl_state *TLS, struct connection *c, long long ms
     assert (skip_type_vector (TYPE_TO_PARAM_1 (vector, TYPE_TO_PARAM (privacy_rule))) >= 0);
     vlogprintf (E_NOTICE, "privacy change update\n");
     break;
+  case CODE_update_user_phone:
+    {
+      int id = fetch_int ();
+      int l = prefetch_strlen ();
+      char *phone = fetch_str (l);
+      tgl_peer_t *U = tgl_peer_get (TLS, TGL_MK_USER (id));
+      if (U && (U->flags & FLAG_CREATED)) {
+         bl_do_user_set_phone (TLS, &U->user, phone, l);
+      }
+    }
+    break;
+
   default:
     vlogprintf (E_ERROR, "Unknown update type %08x\n", op);
     ;
