@@ -22,6 +22,7 @@
 #include "purple.h"
 #include "msglog.h"
 #include "tgp-utils.h"
+#include "tgp-ft.h"
 
 #include <glib.h>
 #include <tgl.h>
@@ -99,7 +100,8 @@ void *connection_data_free (connection_data *conn) {
   
   tgp_g_queue_free_full (conn->pending_reads, pending_reads_free_cb);
   tgp_g_queue_free_full (conn->new_messages, message_text_free);
-  g_list_free (conn->used_images);
+  tgp_g_list_free_full (conn->used_images, used_image_free);
+  tgprpl_xfer_free_all (conn);
   tgl_free_all (conn->TLS);
   free (conn->TLS);
   
