@@ -45,6 +45,14 @@ typedef struct {
   tgl_peer_id_t peer;
 } get_user_info_data;
 
+struct tgp_xfer_send_data {
+  int timer;
+  int done;
+  PurpleXfer *xfer;
+  connection_data *conn;
+  struct tgl_document *document;
+};
+
 struct download_desc {
   get_user_info_data *get_user_info_data;
   void *data;
